//Client interface to restful web services with support for http proxies.
var http = require('http');
var https = require('https');
var url = require('url');
var util = require('util');

function makeClientUrl(urlStr,client) {
    client.url = url.parse(urlStr);
}

var Chunk = exports.chunk = function(request) {
    var self = this;
    self.data = null;
    self.request = request;
    
    self.request.on('response', function(response) {
	self.data=null;
	response.on('data', function(chunk) {
            if(self.data) {
                self.data = self.data+chunk;
            } else {
                self.data = chunk;
            }
	});
    });
}

var Client = exports.endpoint = function(urlStr) {    
    if (!(this instanceof Client)) return new Client();
    if(process.env.HTTP_PROXY) {
	this.proxyHttp = url.parse(process.env.HTTP_PROXY);
    }
    if(process.env.HTTPS_PROXY) {
	this.proxyHttps = url.parse(process.env.HTTPS_PROXY);
    }

    makeClientUrl(urlStr,this);
};

Client.prototype.method = function(method,opts) {
    var options = {
    		method: method,
    		headers: opts.headers
    };
    var protocol =  null;
    if(this.url.protocol==='http:' && this.proxyHttp) {
		protocol=this.proxyHttp.protocol==='http:'?http:https;
		if(this.proxyHttp.hostname)
		    options.host = this.proxyHttp.hostname;
		if(this.proxyHttp.port)
		    options.port = this.proxyHttp.port;
		options.path = urlStr;	
    } else if(this.url.protocol==='https:' && this.proxyHttps) {
		protocol=this.proxyHttps.protocol==='http:'?http:https;
		if(this.proxyHttps.hostname)
		    options.host = this.proxyHttps.hostname;
		if(this.proxyHttps.port)
		    options.port = this.proxyHttps.port;
		options.path = urlStr;	
    } else {
		protocol=this.url.protocol==='http:'?http:https;
		if(this.url.hostname)
		    options.host =this.url.hostname;
		if(this.url.port)
		    options.port = this.url.port;
		if(this.url.pathname)
		    options.path = this.url.pathname;
		if(this.url.search)
		    options.path = options.path+this.url.search;
		if(this.url.hash)
		    options.path = options.path+this.url.hash;
    }

    return protocol.request(options);
};

var mergeHeaders = exports.mergeHeaders = function(headers,headers1) {
    if(headers) {
        for (var key in headers1) {
            if (headers1.hasOwnProperty(key)) {
                headers[key] = headers1[key];
            }
        }
    } else if(headers1) {
        return headers1;
    }
    return headers;
};

Client.prototype.post = function(data,options) {
    if(!options)
        options = {};
    options['headers']=mergeHeaders({'Content-Length':data.length},options.headers);
    var req = this.method('POST',options);  
    req.end(data);
    return req;
};

Client.prototype.get = function(options) {
    var req = this.method('GET',options);  
    req.end();
    return req;
};