// SFA restful services
var client = require("./client");
var util = require("util");
var querystring = require('querystring');
var sfaVersion = "20.0";
var events = require("events");
var urlParser = require('url');
var qs = require('querystring');

exports.listen =  function listen(options){
    options = options || {};
    
    // root required
    if(!options.root)
	throw new Error('listen() root path required');
    if(options.root.length<3)
	throw new Error('listen() root path is empty');
    if(options.root[0]!='/')
	throw new Error('listen() root path must be an absolute path');
    if(options.root.lastIndexOf('/')<(root.length-1))
	throw new Error('listen() root path must end in "/"');
    if(!options.getClientInfo)
	throw new Error('getClientInfo must be defined');
    if(!options.getTokens)
	throw new Error('getTokens must be defined');
    if(!options.putTokens)
	throw new Error('putTokens must be defined');
    options.callback = options.callback || '/_callback';
    options.refresh  = options.refresh  || '/_refresh';
    return function handleRequest(req, res, next) {
	options.path = req.url;
	proxySfaRequest(req, res, next, options);
    };
};

function proxySfaRequest(req,res,next,options) {
    var path = urlParser.parse(options.path);
    // look for root of path, otherwise move to next routing
    if(path.pathname.indexOf(options.root)!=0)
	return next();
    options.path = path.pathname.slice(options.root.length-1);
    options.query = path.search||'';
    options.hash = path.hash||'';
    if(options.path === options.callback)
	return proxySfaHandleTokenCallback(req,res,next,options);
    if(options.path === options.refresh)
	return proxySfaHandleRefreshToken(req,res,next,options);
    return proxySfaGetRequest(req,res,next,options);
};

function proxySfaHandleTokenCallback(req,res,next,options) {
    var query = ~req.url.indexOf('?')?qs.parse(urlParser.parse(req.url).query):{};
    var code = query.code;
    var state = query.state;
    if(code) {
	options.getClientInfo(req, function(err, sfa) {
	    var data = {
		code: code,
		grant_type: "authorization_code",
		client_id: sfa.client_id,
		client_secret: sfa.client_secret,
		redirect_uri: sfa.redirect_uri
	    };
	    var request = new client.endpoint('https://login.salesforce.com/services/oauth2/token').
		post(querystring.stringify(data),{
		    headers: {'Content-Type':'application/x-www-form-urlencoded','Accept': 'application/json'}
		});
	    var chunk = new client.chunk(request);
	    request.on('response', function(response) {
		response.on('end', function() {
		    var tokens = req.session[options.sessionName] = JSON.parse(chunk.data.toString());
		    if(tokens.error) {
			res.writeHead(500,{});
			return res.end(JSON.stringify(tokens));
		    }
		    return options.putTokens(req,tokens,function(err) {
			if(err) {
			    res.writeHead(500,{});
			    return res.end();
			}
			res.writeHead(302,{ Location: state });
			return res.end();
		    });
		});
	    });
	});
    } else {
	res.writeHead(500,{});
	return res.end('Sales Force Automation OAuth Token retrieve');
    }
};

function proxySfaHandleRefreshToken(req,res,next,options) {
    options.getClientInfo(req, function(err, cli) {
	if(err) {
	    res.writeHead(500,{});
	    return res.end();
	}
	return options.getTokens(req,function(err,token) {
	    if(err) {
		res.writeHead(500,{});
		return res.end();
	    }
	    var data = {
		grant_type: "refresh_token",
		client_id: cli.client_id,
		client_secret: cli.client_secret,
		refresh_token: token.refresh_token
	    };
	    var request = new client.
		endpoint('https://login.salesforce.com/services/oauth2/token').
		post(querystring.stringify(data), {
		    headers: {'Content-Type':'application/x-www-form-urlencoded','Accept': 'application/json'}
		});
	    var chunk = new client.chunk(request);
	    return request.on('response', function(response) {
		response.on('end', function() {
		    var tokens = req.session[options.sessionName] = JSON.parse(chunk.data.toString());
		    if(tokens.error) {
			res.writeHead(500,{});
			return res.end(JSON.stringify(tokens));
		    }
		    return options.putTokens(req,tokens,function(err) {
			res.writeHead(200,{});
			return res.end(JSON.stringify({success:true,msg:'Token Refreshed'}));
		    });
		});
	    });
	});
    });
};

function proxySfaGetRequest(req,res,next,options) {
    options.getTokens(req,function(err,sfa) {
	var url = sfa.instance_url+options.path+options.query+options.hash;
        var request = new client.endpoint(url).
	    method(req.method,{
	        headers: client.mergeHeaders(req.headers,{ Authorization: 'OAuth '+sfa.access_token })
	    });
        request.end(req.body);
        request.on('response',function(response) {
	    res.writeHead(response.statusCode,response.headers);
	    response.on('data',function(data) {
	        res.write(data);
	    });
	    response.on('end',function() {
	        res.end();
	    });
        });
        request.on('error',function(e) {
	    console.log('ERROR: '+e);
	    res.writeHead(500,{});
	    res.end('Internal Server Error');
        });
    });
};
